/*
Flip Side Dio

This program reads the state of digital pins 2 through 10.  If any 
of those pins change state it will transmit a byte representing the pin number.
The data is transmitted as hex, not as plain text ascii, so that it will 
only be 1 byte in length.

In order to prevent flooding of the serial port the pin has to have not changed for 
at least 100 milliseconds before it will be transmitted again.

All the defined pins have the internal 20K ohm pullup resistor enabled.  You should tie 
your sensor to ground so that changes can be detected.  With sensors such as 
photocelss you may have to put a resistor in line with the sensor to overcome 
the internal pullup resistor.

On the computer you need to set the baud rate to 115200 to communicate with this sketch.

created 14 March 2011
by Jeremy Auten
http://www.flipsideracing.org
*/

/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Flip Side Dio is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Flip Side Dio.  If not, see <http://www.gnu.org/licenses/>.

Copyright 2011 Jeremy Auten	
*/


/*
When defining the arrays make sure the pinCount matches the number of 
elements in the time, inState and inPins array.  If you are sure you will 
never use certain digital pins you can remove them from this array 
and update pinCount.

Pins 1,2,11,12 and 13 are not used because they are tied to other hardware on
the Arduino Uno.  Pins 1 and 2 can not be used because they are for the 
communication back to the computer.  Pin 13 should not be used because it is 
tied to the LED built on the Arduino Uno board.
*/
int inByte = 0;
unsigned long time[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0};
int inState[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0};
int inPins[] = {
  2, 3, 4, 5, 6, 7, 8, 9, 10};
int pinCount = 9;
unsigned long currentTime;

void setup()
{
  Serial.begin(115200);
  for (int index = 0; index < pinCount; index++) {
    //Set each defined pin as an input and enable the internal pullup resistor
    //The pullup resistor is a 20K resistor that is tied to 5v so we do not have
    //floating inputs.  
    pinMode(inPins[index], INPUT);
    digitalWrite(inPins[index], HIGH);
  }
}

void loop()
{
  //Loop through the array that contains the pin numbers
  for (int index = 0; index < pinCount; index++) {
    //Read the digital input and store it into a variable
    inByte = digitalRead(inPins[index]);
    //Check to see if the state of the pin changed
    if (inByte != inState[index]){
      //Since the pin state changed we check to see how long it was since the last detection
      //If it has not been at least 100 milliseconds since the last detection we ignore it
      //This prevents flooding the serial port with tons of data
      //If it has been greater than 100 ms then we transmit 1 byte of info representing the pin number
      //This will show up as a hex value and not printable ascii
      if (millis() - time[index] > 100) {
        Serial.write(inPins[index]);
      }
      //Store the current state of the pin so we can detect future changes
      inState[index] = inByte;
      time[index] = millis();
    }
  }
}
